
#ifndef _WINCE_H_
#define _WINCE_H_

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdio.h>

#define LIBWINCE_VERSION 20100930

#ifndef PATH_MAX
#define PATH_MAX 255
#endif

/* internal */
void	libwince_init(const char* prog, int debug);

int	is_directory(const char*);
int	is_file(const char*);
int     pathcat(char*, size_t, const char*, ...); /* NULL terminated */

/* unistd.h */
char*	basename(char*);
char*	dirname(char*);
char*	get_current_dir_name(void);
char*	getcwd(char*, size_t);
int	chdir(const char *path);

/* stdlib.h */
int	putenv(const char*);
int	clearenv(void);
char*	getenv(const char*);
int	setenv(const char*, const char*, int);
int	unsetenv(const char*);
int	system(const char*);

/* stdio.h */
FILE*	tmpfile(void);
char*	tmpnam(char*);

/* time.h */
#ifndef clock_t
typedef long clock_t;
#endif
clock_t	clock(void);

/* errno.h */
#define EPERM		1	/* Operation not permitted */
#define	ENOFILE		2	/* No such file or directory */
#define	ENOENT		2
#define	ESRCH		3	/* No such process */
#define	EINTR		4	/* Interrupted function call */
#define	EIO		5	/* Input/output error */
#define	ENXIO		6	/* No such device or address */
#define	E2BIG		7	/* Arg list too long */
#define	ENOEXEC		8	/* Exec format error */
#define	EBADF		9	/* Bad file descriptor */
#define	ECHILD		10	/* No child processes */
#define	EAGAIN		11	/* Resource temporarily unavailable */
#define	ENOMEM		12	/* Not enough space */
#define	EACCES		13	/* Permission denied */
#define	EFAULT		14	/* Bad address */
/* 15 - Unknown Error */
#define	EBUSY		16	/* strerror reports "Resource device" */
#define	EEXIST		17	/* File exists */
#define	EXDEV		18	/* Improper link (cross-device link?) */
#define	ENODEV		19	/* No such device */
#define	ENOTDIR		20	/* Not a directory */
#define	EISDIR		21	/* Is a directory */
#define	EINVAL		22	/* Invalid argument */
#define	ENFILE		23	/* Too many open files in system */
#define	EMFILE		24	/* Too many open files */
#define	ENOTTY		25	/* Inappropriate I/O control operation */
/* 26 - Unknown Error */
#define	EFBIG		27	/* File too large */
#define	ENOSPC		28	/* No space left on device */
#define	ESPIPE		29	/* Invalid seek (seek on a pipe?) */
#define	EROFS		30	/* Read-only file system */
#define	EMLINK		31	/* Too many links */
#define	EPIPE		32	/* Broken pipe */
#define	EDOM		33	/* Domain error (math functions) */
#define	ERANGE		34	/* Result too large (possibly too small) */
/* 35 - Unknown Error */
#define	EDEADLOCK	36	/* Resource deadlock avoided (non-Cyg) */
#define	EDEADLK		36
/* 37 - Unknown Error */
#define	ENAMETOOLONG	38	/* Filename too long (91 in Cyg?) */
#define	ENOLCK		39	/* No locks available (46 in Cyg?) */
#define	ENOSYS		40	/* Function not implemented (88 in Cyg?) */
#define	ENOTEMPTY	41	/* Directory not empty (90 in Cyg?) */
#define	EILSEQ		42	/* Illegal byte sequence */

int*	get_errno(void);
#define errno (*get_errno())

const char *strerror(int);

/* locale.h */
#define LC_ALL          0
#define LC_COLLATE      1
#define LC_CTYPE        2
#define LC_MONETARY     3
#define LC_NUMERIC      4
#define LC_TIME         5
#define LC_MIN          LC_ALL
#define LC_MAX          LC_TIME

const char*	setlocale(int, const char*);





/* windows.h */
#include <windows.h>

LPWSTR		cstr_to_LPWSTR(const char*);
char*		LPWSTR_to_cstr(const LPWSTR);
wchar_t*	char2wchar(const char*);
char*		wchar2char(const wchar_t*);

HMODULE		LoadLibraryA(const char *filename);
DWORD		FormatMessageA(unsigned long, LPCVOID, DWORD, DWORD, LPSTR, DWORD, va_list *);
DWORD		GetModuleFileNameA(HMODULE, LPSTR, DWORD);
FARPROC		GetProcAddressA(HINSTANCE, const char*);

#ifdef GetLocaleInfoA
#undef GetLocaleInfoA
#endif

int		GetLocaleInfoA(LCID, LCTYPE, LPSTR, int);

/* fix char */
#define is_ll_hh(a, ll, hh) (ll <= a && hh >= a)

#define is_00_08(a) is_ll_hh(a, 0x00, 0x08)
#define is_09_0d(a) is_ll_hh(a, 0x09, 0x0d)
#define is_0e_1f(a) is_ll_hh(a, 0x0e, 0x1f)
#define is_21_2f(a) is_ll_hh(a, 0x21, 0x2f)
#define is_30_39(a) is_ll_hh(a, 0x30, 0x39)
#define is_3a_40(a) is_ll_hh(a, 0x3a, 0x40)
#define is_41_46(a) is_ll_hh(a, 0x41, 0x46)
#define is_47_5a(a) is_ll_hh(a, 0x47, 0x5a)
#define is_5b_60(a) is_ll_hh(a, 0x5b, 0x60)
#define is_61_66(a) is_ll_hh(a, 0x61, 0x66)
#define is_67_7a(a) is_ll_hh(a, 0x67, 0x7a)
#define is_7b_7e(a) is_ll_hh(a, 0x7b, 0x7e)

#define is_20(a)    (0x20 == a)
#define is_7f(a)    (0x7f == a)

#undef iscntrl
#define iscntrl(a) (is_00_08(a) || is_09_0d(a) || is_0e_1f(a) || is_7f(a))

#undef isspace
#define isspace(a) (is_09_0d(a) || is_20(a))

#undef isupper
#define isupper(a) (is_41_46(a) || is_47_5a(a))

#undef isslower
#define isslower(a) (is_61_66(a) || is_67_7a(a))

#undef isalpha
#define isalpha(a) (is_41_46(a) || is_47_5a(a) || is_61_66(a) || is_67_7a(a))

#undef isdigit
#define isdigit(a) (is_30_39(a))

#undef isxdigit
#define isxdigit(a) (is_30_39(a) || is_41_46(a) || is_61_66(a))

#undef isalnum
#define isalnum(a) (isalpha(a) || isdigit(a))

#undef ispunct
#define ispunct(a) (is_21_2f(a) || is_3a_40(a) || is_5b_60(a) || is_7b_7e(a))

#undef isgraph
#define isgraph(a) (ispunct(a) || isalnum(a))

#undef isprint
#define isprint(a) (isgraph(a) || is_20(a))

#ifdef  __cplusplus
}
#endif

#endif /* _WINCE_H_ */
